module cartridge()
{
	$fn = 50;

	module fillet(r, h) {
		translate([0, 0, -h/2])
			difference() {
				cube([r, r, h]);
				cylinder(r = r, h = h);
			}
	}

	/* Cartridge dimensions. */

	width = 89.0;
	height = 68.5;
	depth = 16.5;

	/* Details. */

	front_label_width = 83.0;
	front_label_height = 46.0;
	front_label_depth = 1.0;
	front_label_offset_from_bottom = 19.5;
	top_label_width = front_label_width;
	top_label_height = 11.5;
	top_label_depth = front_label_depth;
	top_label_offset_from_front = 2.5;

	/* Side thicknesses. */

	front = 2;
	back = 3.5;
	top = 3;
	left = 2;
	right = 2;

	/* Rounding/fillet offset, radius and additional length.
	The radius needs to be slightly larger than the offset
	in order to avoid geometry problems. */

	ro = 1;
	rr = 1.01;
	extra = 0;

	difference() {

		/* The cartridge surfaces. */

		union() {
			translate([0, -depth / 2 + front / 2, 0])
				cube([width, front, height], center = true);
			translate([0, depth / 2 - back / 2, 0])
				cube([width, back, height], center = true);
			translate([-width / 2 + left / 2, 0, 0])
				cube([left, depth, height], center = true);
			translate([width / 2 - right / 2, 0, 0])
				cube([right, depth, height], center = true);
			translate([0, 0, height / 2 - top / 2])
				cube([width, depth, top], center = true);
		}

		/* Label insets. */

		union() {

			/* Front label. */

			translate([-front_label_width / 2, -depth / 2,
				front_label_offset_from_bottom - height / 2])
				cube([front_label_width, front_label_depth,
					front_label_height]);

			/* Top label. */

			translate([-top_label_width / 2,
				-depth / 2 + top_label_offset_from_front,
				height / 2 - top_label_depth])
				cube([top_label_width, top_label_height,
					top_label_depth]);
		}

		/* Fillets to round off the edges. */

		union() {

			/* Top left and right rounding. */
		
			translate([-width / 2 + ro, 0, height / 2 - ro])
				rotate([0, 0, 180])
					rotate([90, 0, 0])
						fillet(rr, depth + extra);
			translate([width / 2 - ro, 0, height / 2 - ro])
				rotate([90, 0, 0])
					fillet(rr, depth + extra);
		
			/* Top back and front rounding. */
		
			translate([0, depth / 2 - ro, height / 2 - ro])
				rotate([0, -90, 0])
					fillet(rr, width + extra);
			translate([0, -depth / 2 + ro, height / 2 - ro])
				rotate([0, 0, 180])
					rotate([0, -90, 0])
						fillet(rr, width + extra);
		
			/* Edge rounding. */
		
			translate([width / 2 - ro, depth / 2 - ro, 0])
				fillet(rr, height + extra);
			translate([-width / 2 + ro, depth / 2 - ro, 0])
				rotate([0, 0, 90])
					fillet(rr, height + extra);
			translate([-width / 2 + ro, -depth / 2 + ro, 0])
				rotate([0, 0, 180])
					fillet(rr, height + extra);
			translate([width / 2 - ro, -depth / 2 + ro, 0])
				rotate([0, 0, 270])
					fillet(rr, height + extra);
		}
	}
}

cartridge();

